/*
 * Copyright (c) 1999, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug 4102881
 * @summary Ensure multiple selection Lists have horizontal scrollbars when necessary
 * @library /java/awt/regtesthelpers
 * @build PassFailJFrame
 * @run main/manual MultiSelectionListHorizScrollbar
*/

import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.List;

public class MultiSelectionListHorizScrollbar {

    private static final String INSTRUCTIONS = """
         Resize the frame so that the lists are not wide enough
         to fully display the lines of text they contain.
         Once the lists are in this state, press pass
         if both lists display an horizontal scrollbar. Otherwise press fail.""";

    public static void main(String[] args) throws Exception {
        PassFailJFrame.builder()
                .title("MultiSelectionListHorizScrollbar Instructions")
                .instructions(INSTRUCTIONS)
                .rows((int) INSTRUCTIONS.lines().count() + 2)
                .columns(35)
                .testUI(MultiSelectionListHorizScrollbar::createTestUI)
                .build()
                .awaitAndCheck();
    }

    private static Frame createTestUI() {
        Frame frame = new Frame("MultiSelectionListHorizScrollbar Frame");
        List singleList = new List(3);
        List multiList = new List(3, true);

        frame.setLayout(new GridLayout(1, 2));
        frame.add(singleList);
        frame.add(multiList);

        singleList.addItem("This is the 1st item in the list! Does it scroll horizontally??");
        singleList.addItem("This is the 2nd item in the list! Does it scroll horizontally??");
        singleList.addItem("This is the 4th item in the list! Does it scroll horizontally??");
        singleList.addItem("This is the 5th item in the list! Does it scroll horizontally??");
        singleList.addItem("This is the 6th item in the list! Does it scroll horizontally??");

        multiList.addItem("This is the 1st item in the list! Does it scroll horizontally??");
        multiList.addItem("This is the 2nd item in the list! Does it scroll horizontally??");
        multiList.addItem("This is the 4th item in the list! Does it scroll horizontally??");
        multiList.addItem("This is the 5th item in the list! Does it scroll horizontally??");
        multiList.addItem("This is the 6th item in the list! Does it scroll horizontally??");

        frame.pack();
        return frame;
    }
}
